"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRollupConfig = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("./utils");
const constants_1 = require("./constants");
const rollup_plugin_terser_1 = require("rollup-plugin-terser");
const core_1 = require("@babel/core");
const plugin_commonjs_1 = tslib_1.__importDefault(require("@rollup/plugin-commonjs"));
const plugin_json_1 = tslib_1.__importDefault(require("@rollup/plugin-json"));
const plugin_replace_1 = tslib_1.__importDefault(require("@rollup/plugin-replace"));
const plugin_node_resolve_1 = tslib_1.__importStar(require("@rollup/plugin-node-resolve"));
const rollup_plugin_sourcemaps_1 = tslib_1.__importDefault(require("rollup-plugin-sourcemaps"));
const rollup_plugin_typescript2_1 = tslib_1.__importDefault(require("rollup-plugin-typescript2"));
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const extractErrors_1 = require("./errors/extractErrors");
const babelPluginLujc_1 = require("./babelPluginLujc");
const errorCodeOpts = {
    errorMapFilePath: constants_1.paths.appErrorsJson,
};
let shebang = {};
async function createRollupConfig(opts, outputNum) {
    const findAndRecordErrorCodes = await (0, extractErrors_1.extractErrors)(Object.assign(Object.assign({}, errorCodeOpts), opts));
    const isEsm = opts.format.includes('es') || opts.format.includes('esm');
    const shouldMinify = opts.minify !== undefined
        ? opts.minify
        : opts.env === 'production' || isEsm;
    let formatString = ['esm', 'cjs'].includes(opts.format) ? '' : opts.format;
    let fileExtension = opts.format === 'esm' ? 'mjs' : 'cjs';
    const outputName = [
        `${constants_1.paths.appDist}/${(0, utils_1.safePackageName)(opts.name)}`,
        formatString,
        opts.env,
        shouldMinify ? 'min' : '',
        fileExtension,
    ]
        .filter(Boolean)
        .join('.');
    const tsconfigPath = opts.tsconfig || constants_1.paths.tsconfigJson;
    const tsconfigJSON = typescript_1.default.readConfigFile(tsconfigPath, typescript_1.default.sys.readFile).config;
    const tsCompilerOptions = typescript_1.default.parseJsonConfigFileContent(tsconfigJSON, typescript_1.default.sys, './').options;
    return {
        input: opts.input,
        external: (id) => {
            if (id.startsWith('regenerator-runtime')) {
                return false;
            }
            return (0, utils_1.external)(id);
        },
        treeshake: {
            propertyReadSideEffects: false,
        },
        output: {
            file: outputName,
            format: opts.format,
            freeze: false,
            esModule: Boolean(tsCompilerOptions === null || tsCompilerOptions === void 0 ? void 0 : tsCompilerOptions.esModuleInterop),
            name: opts.name || (0, utils_1.safeVariableName)(opts.name),
            sourcemap: true,
            globals: {
                react: 'React',
                'react-native': 'ReactNative',
                'lodash-es': 'lodashEs',
                'lodash/fp': 'lodashFp',
            },
            exports: 'named',
        },
        plugins: [
            !!opts.extractErrors && {
                async transform(code) {
                    try {
                        await findAndRecordErrorCodes(code);
                    }
                    catch (e) {
                        return null;
                    }
                    return { code, map: null };
                },
            },
            (0, plugin_node_resolve_1.default)({
                mainFields: [
                    'module',
                    'main',
                    opts.target !== 'node' ? 'browser' : undefined,
                ].filter(Boolean),
                extensions: [...plugin_node_resolve_1.DEFAULTS.extensions, '.cjs', '.mjs', '.jsx'],
            }),
            (0, plugin_commonjs_1.default)({
                include: opts.format === 'umd'
                    ? /\/node_modules\//
                    : /\/regenerator-runtime\//,
            }),
            (0, plugin_json_1.default)(),
            {
                transform(code) {
                    let reg = /^#!(.*)/;
                    let match = code.match(reg);
                    shebang[opts.name] = match ? '#!' + match[1] : '';
                    code = code.replace(reg, '');
                    return {
                        code,
                        map: null,
                    };
                },
            },
            (0, rollup_plugin_typescript2_1.default)({
                typescript: typescript_1.default,
                tsconfig: opts.tsconfig,
                tsconfigDefaults: {
                    exclude: [
                        '**/*.spec.ts',
                        '**/*.test.ts',
                        '**/*.spec.tsx',
                        '**/*.test.tsx',
                        // TS defaults below
                        'node_modules',
                        'bower_components',
                        'jspm_packages',
                        constants_1.paths.appDist,
                    ],
                    compilerOptions: {
                        sourceMap: true,
                        declaration: true,
                        jsx: 'react',
                    },
                },
                tsconfigOverride: {
                    compilerOptions: Object.assign({ target: 'esnext' }, (outputNum > 0
                        ? { declaration: false, declarationMap: false }
                        : {})),
                },
                check: !opts.transpileOnly && outputNum === 0,
                useTsconfigDeclarationDir: Boolean(tsCompilerOptions === null || tsCompilerOptions === void 0 ? void 0 : tsCompilerOptions.declarationDir),
            }),
            (0, babelPluginLujc_1.babelPluginLujc)({
                exclude: 'node_modules/**',
                extensions: [...core_1.DEFAULT_EXTENSIONS, 'ts', 'tsx'],
                passPerPreset: true,
                custom: {
                    targets: opts.target === 'node' ? { node: '14' } : undefined,
                    extractErrors: opts.extractErrors,
                    format: opts.format,
                },
                babelHelpers: 'bundled',
            }),
            opts.env !== undefined &&
                (0, plugin_replace_1.default)({
                    preventAssignment: true,
                    'process.env.NODE_ENV': JSON.stringify(opts.env),
                }),
            (0, rollup_plugin_sourcemaps_1.default)(),
            shouldMinify &&
                (0, rollup_plugin_terser_1.terser)({
                    output: { comments: false },
                    compress: {
                        keep_infinity: true,
                        pure_getters: true,
                        passes: 10,
                    },
                    ecma: opts.legacy ? 5 : 2020,
                    module: isEsm,
                    toplevel: opts.format === 'cjs' || isEsm,
                    warnings: true,
                }),
            {
                renderChunk: async (code, chunk) => {
                    if (chunk.exports.includes('default') || !isEsm) {
                        return null;
                    }
                    return {
                        code: `${code}\nexport default {};`,
                        map: null,
                    };
                },
            },
        ],
    };
}
exports.createRollupConfig = createRollupConfig;
